@@ -8,7 +8,7 @@
 //! assert!(tracker.acquire(&CONFIG, 1).is_err());
 //! ```
 
-#[cfg(feature="use_serde")] mod serde;
+#[cfg(feature="serde")] mod serde;
 
 /// Config for rate limiting.
 ///
@@ -106,6 +106,31 @@ impl Tracker {
 		}
 	}
 
+	/// Create a tracker that has the specified capacity.
+	///
+	/// Equivalent to `Tracker::with_capacity_at(config, capacity, std::time::SystemTime::now())`.
+	pub fn with_capacity(
+		config: &Config,
+		capacity: u32,
+	) -> Self {
+		Self::with_capacity_at(config, capacity, std::time::SystemTime::now())
+	}
+
+	/// Create a tracker that has the specified capacity at the specified time.
+	pub fn with_capacity_at(
+		config: &Config,
+		capacity: u32,
+		now: std::time::SystemTime
+	) -> Self {
+		if capacity >= config.burst {
+			return Self::full()
+		}
+
+		Tracker{
+			last: Some(now - config.rate * capacity),
+		}
+	}
+
 	/// Returns the tokens currently available.
 	///
 	/// Equivalent to `Tracker::capacity_at(std::time::SystemTime::now())`.
@@ -273,6 +298,39 @@ fn test_rate_limit_active_capacity() {
 	assert_eq!(t.capacity_at(&cfg, std::time::UNIX_EPOCH + std::time::Duration::from_secs(50)), 10);
 }
 
+#[test]
+fn test_with_capacity() {
+	let cfg = Config::new(std::time::Duration::from_secs(1), 10);
+	let now = std::time::SystemTime::now();
+
+	// Basic operation.
+	let t = Tracker::with_capacity_at(&cfg, 5, now);
+	assert_eq!(t.capacity_at(&cfg, now), 5);
+	assert_eq!(t.clone().acquire_at(&cfg, 5, now), Ok(()));
+	assert_eq!(
+		t.clone().acquire_at(&cfg, 6, now),
+		Err(Denied::TooEarly(TooEarly{
+			next: now + std::time::Duration::from_secs(1),
+		})));
+
+	// Creating empty works as expected.
+	let t = Tracker::with_capacity_at(&cfg, 0, now);
+	assert_eq!(t.capacity_at(&cfg, now), 0);
+	assert_eq!(
+		t.clone().acquire_at(&cfg, 1, now),
+		Err(Denied::TooEarly(TooEarly{
+			next: now + std::time::Duration::from_secs(1),
+		})));
+
+	// Full or over-full trackers optimize to untracked.
+	let t = Tracker::with_capacity_at(&cfg, 10, now);
+	assert_eq!(t.last, None);
+	let t = Tracker::with_capacity_at(&cfg, 11, now);
+	assert_eq!(t.last, None);
+	let t = Tracker::with_capacity_at(&cfg, 201, now);
+	assert_eq!(t.last, None);
+}
+
 #[test]
 fn test_rate_limit_active_acquire() {
 	let cfg = Config::new(std::time::Duration::from_secs(1), 10);
